/*
 * http_config.c: auxillary functions for reading httpd's config file
 * and converting filenames into a namespace
 *
 * Rob McCool 
 * 
 */

#include "httpd.h"


/* Configuration globals */
int standalone;
int port;
uid_t user_id;
gid_t group_id;
char server_root[MAX_STRING_LEN];
char error_fname[MAX_STRING_LEN];
char xfer_fname[MAX_STRING_LEN];
char bindir[MAX_STRING_LEN];
char index_name[MAX_STRING_LEN];
char document_root[MAX_STRING_LEN];
char server_admin[MAX_STRING_LEN];
int no_indexes;

void process_server_config() {
    FILE *cfg;
    char l[MAX_STRING_LEN],w[MAX_STRING_LEN],fname[MAX_STRING_LEN];
    int n=0;

    make_full_path(server_root,SERVER_CONFIG_FILE,fname);
    if(!(cfg = fopen(fname,"r"))) {
        fprintf(stderr,"httpd: could not open server config. file %s\n",fname);
        perror("fopen");
        exit(1);
    }

    /* Parse server config file. Remind me to learn yacc. */
    while(!(getline(l,MAX_STRING_LEN,cfg,0))) {
        ++n;
        if((l[0] != '#') && (l[0] != '\0')) {
            getword(w,l,' ');
            
            if(!strcmp(w,"ServerType")) {
                getword(w,l,'\0');
                if(!strcmp(w,"inetd")) standalone=0;
                else if(!strcmp(w,"standalone")) standalone=1;
                else {
                    fprintf(stderr,"Syntax error on line %d of %s:\n",n,fname);
                    fprintf(stderr,"ServerType is either inetd or standalone.\n");
                    exit(1);
                }
            }
            else if(!strcmp(w,"Port")) {
                getword(w,l,'\0');
                port = atoi(w);
            } 
            else if(!strcmp(w,"User")) {
                getword(w,l,'\0');
                user_id = uname2id(w);
            } 
            else if(!strcmp(w,"Group")) {
                getword(w,l,'\0');
                group_id = gname2id(w);
            }
            else if(!strcmp(w,"ServerAdmin")) {
                getword(w,l,'\0');
                strcpy(server_admin,w);
            }
            else if(!strcmp(w,"ServerRoot")) {
                getword(w,l,'\0');
                if(!is_directory(w)) {
                    fprintf(stderr,"Syntax error on line %d of %s:\n",n,fname);
                    fprintf(stderr,"%s is not a valid directory.\n",w);
                    exit(1);
                }
                strcpy(server_root,w);
            } 
            else if(!strcmp(w,"ErrorLog")) {
                getword(w,l,'\0');
                if(w[0] != '/')
                    make_full_path(server_root,w,error_fname);
                else 
                    strcpy(error_fname,w);
            } 
            else if(!strcmp(w,"TransferLog")) {
                getword(w,l,'\0');
                if(w[0] != '/')
                    make_full_path(server_root,w,xfer_fname);
                else strcpy(xfer_fname,w);
            } 
            else if(!strcmp(w,"BinDir")) {
                getword(w,l,'\0');
                if(w[0] != '/')
                    make_full_path(server_root,w,bindir);
                else strcpy(bindir,w);
                if(!is_directory(bindir)) {
                    fprintf(stderr,"Syntax error on line %d of %s:\n",n,fname);
                    fprintf(stderr,"%s is not a valid directory.\n",bindir);
                    exit(1);
                }
            } 
            else if(!strcmp(w,"DirectoryIndex")) {
                getword(w,l,'\0');
                strcpy(index_name,w);
            } 
            else if(!strcmp(w,"DocumentRoot")) {
                getword(w,l,'\0');
                if(!is_directory(w)) {
                    fprintf(stderr,"Syntax error on line %d of %s:\n",n,fname);
                    fprintf(stderr,"%s is not a valid directory.\n",w);
                    exit(1);
                }
                strcpy(document_root,w);
            } 
            else if(!strcmp(w,"Indexes")) {
                getword(w,l,'\0');
                if(!strcmp(w,"off"))
                    no_indexes=1;
                else if(!strcmp(w,"on"))
                    no_indexes=0;
                else {
                    fprintf(stderr,"Syntax error on line %d of %s:\n",n,fname);
                    fprintf(stderr,"Indexes must be on or off.\n");
                    exit(1);
                }
            }
            else if(!strcmp(w,"Alias")) {
                char w2[MAX_STRING_LEN];
                
                getword(w,l,' ');
                getword(w2,l,'\0');
                if((w[0] == '\0') || (w2[0] == '\0')) {
                    fprintf(stderr,"Syntax error on line %d of %s:\n",n,fname);
                    fprintf(stderr,"Alias must be followed by a fakename, one space, then a realname.\n");
                    exit(1);
                }                
                add_alias(w,w2);
            }
            else if(!strcmp(w,"Deny")) {
                getword(w,l,'\0');
                add_alias(w,DENY_NAME);
            }
            else {
                fprintf(stderr,"Syntax error on line %d of %s:\n",n,fname);
                fprintf(stderr,"Unknown keyword %s.\n",w);
                exit(1);
            }
        }
    }
}

void dump_config() {
    printf("Configuration:\n");
    printf("Server is running %s.\n",(standalone ? "standalone" : "inetd"));
    printf("Listening to port %d.\n",port);
    printf("Running as user %d, group %d.\n",user_id,group_id);
    printf("Server admin is %s.\n",server_admin);
    printf("Server root is at %s.\n",server_root);
    printf("Error log is %s\n",error_fname);
    printf("Transfer log is %s\n",xfer_fname);
    printf("Binaries are in %s.\n",bindir);
    printf("Index files are called %s.\n",index_name);
    printf("Documents are in %s.\n",document_root);
    printf("Directory indexing is %s.\n",(no_indexes ? "inactive" : "active"));
    dump_aliases();
}

void read_config()
{
    /* Init. defaults */
    standalone = 1;
    port = DEFAULT_PORT;
    user_id = uname2id(DEFAULT_USER);
    group_id = gname2id(DEFAULT_GROUP);
    strcpy(server_admin,DEFAULT_ADMIN);
    /* ServerRoot set in httpd.c */
    make_full_path(server_root,DEFAULT_ERRORLOG,error_fname);
    make_full_path(server_root,DEFAULT_XFERLOG,xfer_fname);
    make_full_path(server_root,DEFAULT_BIN,bindir);
    strcpy(index_name,DEFAULT_INDEX);
    strcpy(document_root,DOCUMENT_LOCATION);
    no_indexes=0;
    reset_aliases();
    process_server_config();
    add_alias("/htbin",bindir);
#if 0
    dump_config();
#endif
    init_mime();
#if 0
    dump_types();
#endif
}
