/*
 * http_mime.c: Sends/gets MIME headers for requests
 * 
 * Rob McCool
 * 
 */


#include "httpd.h"

struct mime_ext {
    char *ext;
    char *ct;
    struct mime_ext *next;
};

/* Hash table */
struct mime_ext *types[26];

static char content_type[MAX_STRING_LEN];
static char last_modified[MAX_STRING_LEN];
static int content_length;


void hash_insert(struct mime_ext *me) {
    register int i = me->ext[0] - 'a';
    register struct mime_ext *p, *q;

    if(!(q=types[i])) {
        types[i]=me;
        return;
    }
    if((!(p=q->next)) && (strcmp(q->ext,me->ext) >= 0)) {
        types[i]=me;
        me->next=q;
        return;
    }
    while(p) {
        if(strcmp(p->ext,me->ext) >= 0) break;
        q=p;
        p=p->next;
    }
    me->next=p;
    q->next=me;
}

void init_mime() {
    char fn[MAX_STRING_LEN],l[MAX_STRING_LEN],w[MAX_STRING_LEN],*ct;
    FILE *f;
    register struct mime_ext *me;
    register int x,y;

    make_full_path(server_root,MIME_TYPES_FILE,fn);
    if(!(f = fopen(fn,"r"))) {
        fprintf(stderr,"httpd: could not open mime types file %s\n",fn);
        perror("fopen");
        exit(1);
    }

    while(!(getline(l,MAX_STRING_LEN,f,0))) {
        getword(w,l,' ');
        if(l[0] == '\0') continue;
        ct = (char *)malloc(sizeof(char) * (strlen(w) + 1));
        strcpy(ct,w);

        y=0;
        while(l[0]) {
            getword(w,l,' ');
            if(w[0] && (w[0] != ' ')) {
                ++y;
                me = (struct mime_ext *)malloc(sizeof(struct mime_ext));
                me->ext = (char *)malloc(sizeof(char) * (strlen(w)+1));
                for(x=0;w[x];x++)
                    me->ext[x] = tolower(w[x]);
                me->ct=ct;
                me->next=NULL;
                hash_insert(me);
            }
        }
        if(!y) free(ct);
    }

    fclose(f);
}

void dump_types() {
    struct mime_ext *p;
    register int x;

    for(x=0;x<26;x++) {
        p=types[x];
        while(p) {
            printf("ext %s: %s\n",p->ext,p->ct);
            p=p->next;
        }
    }   
}

int is_content_type(char *type) {
    return(!strcmp(content_type,type));
}

void set_content_type(char *fn) {
    int i;
    struct mime_ext *p;

    if((i=rind(fn,'.')) < 0)
        return;

    p=types[tolower(fn[++i]) - 'a'];
    while(p) {
        if(!strcasecmp(&fn[i],p->ext)) {
            strcpy(content_type,p->ct);
            return;
        }
        p=p->next;
    }
    return;
}

void scan_content_type(FILE *f) {
    char l[MAX_STRING_LEN];
    char w[MAX_STRING_LEN];

    while(1) {
        getline(l,MAX_STRING_LEN,f,0);
        if(l[0] == '\0') return;
        getword(w,l,':');
        if(!strcmp(w,"Content-type"))
            getword(content_type,l,'\0');
    }   
}
void set_content_length(int l) {
    content_length = l;
}

void set_last_modified(time_t t) {
    struct tm *tt;
    char *ts;

    tt = gmtime(&t);
    ts = asctime(tt);
    ts[strlen(ts) - 1] = '\0';
    strcpy(last_modified,ts);
}

void get_mime_headers(FILE *fd) {
    char l[MAX_STRING_LEN];
    char w[MAX_STRING_LEN];

    content_type[0] = '\0';
    last_modified[0] = '\0';
    content_length = -1;

    while(1) {
        getline(l,MAX_STRING_LEN,fd,TIMEOUT);
        getword(w,l,':');
        if(!strcmp(w,"Content-type"))
            set_content_type(l);
        else if(!l[0])
            return;
    }
}

void send_mime_headers(FILE *fd) {
    fprintf(fd,"%s %s%c",SERVER_PROTOCOL,"200 OK",LF);
    fprintf(fd,"Date: %s GMT%c",get_gmt(),LF);
    fprintf(fd,"Server: %s%c",SERVER_VERSION,LF);
    fprintf(fd,"MIME-version: 1.0%c",LF);
    if(content_type[0])
        fprintf(fd,"Content-type: %s%c",content_type,LF);
    if(last_modified[0])
        fprintf(fd,"Last-modified: %s GMT%c",last_modified,LF);
/*    if(content_length >= 0) 
        fprintf(fd,"Content-length: %d%c",content_length,LF); */

    fprintf(fd,"%c",LF);
}
